@@ -26,10 +26,10 @@ from pathlib import Path
 # So we import it here first, so that the testsuite doesn't appear to fail.
 import subprocess
 
-from concurrent.futures import ThreadPoolExecutor
+import asyncio
 
 from testutil import getStdout, str_warn, str_info, print_table, shorten_metric_name
-from testglobals import getConfig, ghc_env, getTestRun, TestConfig, \
+from testglobals import getConfig, ghc_env, TestConfig, t, \
                         TestOptions, brokens, PerfMetric
 from my_typing import TestName
 from perf_notes import MetricChange, GitRef, inside_git_repo, is_worktree_dirty, format_perf_stat, get_abbrev_hash_length, is_commit_hash
@@ -71,6 +71,7 @@ parser.add_argument("--config", action='append', help="config field")
 parser.add_argument("--rootdir", action='append', help="root of tree containing tests (default: .)")
 parser.add_argument("--metrics-file", help="file in which to save (append) the performance test metrics. If omitted, git notes will be used.")
 parser.add_argument("--summary-file", help="file in which to save the (human-readable) summary")
+parser.add_argument("--target-wrapper", help="wrapper executable to use when executing binaries compiled for the target")
 parser.add_argument("--no-print-summary", action="store_true", help="should we print the summary?")
 parser.add_argument("--only", action="append", help="just this test (can be give multiple --only= flags)")
 parser.add_argument("--way", action="append", help="just this way")
@@ -101,7 +102,8 @@ if args.e:
 
 if args.config_file:
     for arg in args.config_file:
-        exec(open(arg).read())
+        with io.open(arg) as f:
+            exec(f.read())
 
 if args.config:
     for arg in args.config:
@@ -118,6 +120,7 @@ hasMetricsFile = config.metrics_file is not None
 config.summary_file = args.summary_file
 config.no_print_summary = args.no_print_summary
 config.baseline_commit = args.perf_baseline
+config.target_wrapper = args.target_wrapper
 
 if args.top:
     config.top = args.top
@@ -225,7 +228,7 @@ else:
         h.close()
         if v != '':
             # If it does then use the first utf8 locale that is available
-            h = os.popen('locale -a | grep -i "utf8\|utf-8" 2>/dev/null', 'r')
+            h = os.popen(r'locale -a | grep -i "utf8\|utf-8" 2>/dev/null', 'r')
             v = h.readline().strip()
             h.close()
             if v != '':
@@ -306,7 +309,7 @@ if windows:
                 path = format_path(path)
                 ghc_env['PATH'] = os.pathsep.join([path, ghc_env.get("PATH", "")])
 
-testopts_local.x = TestOptions()
+testopts_ctx_var.set(TestOptions())
 
 # if timeout == -1 then we try to calculate a sensible value
 if config.timeout == -1:
@@ -336,8 +339,6 @@ t_files = list(findTFiles(config.rootdirs))
 
 print('Found', len(t_files), '.T files...')
 
-t = getTestRun() # type: TestRun
-
 # Avoid cmd.exe built-in 'date' command on Windows
 t.start_time = datetime.datetime.now()
 
@@ -350,8 +351,6 @@ except Exception as e:
     print('Failed to detect CPU features: ', e)
 
 sys.stdout.flush()
-# we output text, which cannot be unbuffered
-sys.stdout = os.fdopen(sys.__stdout__.fileno(), "w")
 
 if config.local:
     tempdir = ''
@@ -485,25 +484,28 @@ if config.list_broken:
 else:
     # Now run all the tests
     try:
-        with ThreadPoolExecutor(max_workers=config.threads) as executor:
+        async def run_parallelTests():
+            sem = asyncio.Semaphore(config.threads)
+            ts = []
+
             for oneTest in parallelTests:
                 if stopping():
                     break
-                oneTest(executor)
+                ts.append(oneTest(sem))
 
-            # wait for parallel tests to finish
-            if not stopping():
-                executor.shutdown(wait=True)
+            return await asyncio.gather(*ts)
+
+        # wait for parallel tests to finish
+        asyncio.run(run_parallelTests())
 
         # Run the following tests purely sequential
-        with ThreadPoolExecutor(max_workers=1) as executor:
+        async def run_aloneTests():
             for oneTest in aloneTests:
                 if stopping():
                     break
-                oneTest(executor)
+                await oneTest(None)
 
-            if not stopping():
-                executor.shutdown(wait=True)
+        asyncio.run(run_aloneTests())
 
     except KeyboardInterrupt:
         pass
@@ -524,13 +526,13 @@ else:
 
             groups[m.stat.metric].append(m)
 
-        for metric_name, stats in groups.items():
+        for metric_name, stats in groups.items(): # type: ignore
             heading = 'Metrics: %s' % metric_name
             print()
             print(heading)
             print('-' * len(heading))
             print()
-            tabulate_metrics(stats)
+            tabulate_metrics(stats) # type: ignore
     else:
         print("\nNone collected.")
     print("")
@@ -605,11 +607,13 @@ else:
 
     if args.junit:
         junit(t).write(args.junit)
+        args.junit.close()
 
     if config.only_report_hadrian_deps:
       print("WARNING - skipping all tests and only reporting required hadrian dependencies:", config.hadrian_deps)
       for d in config.hadrian_deps:
         print(d,file=config.only_report_hadrian_deps)
+      config.only_report_hadrian_deps.close()
 
 if len(t.unexpected_failures) > 0 or \
    len(t.unexpected_stat_failures) > 0 or \
